.result-row .result-name{width:240px}.text-truncate-right{direction:rtl}.table-xy-overflow-scroll{display:block;width:100%;max-height:400px;overflow-x:auto;overflow-y:auto}.breadcrumb-link{width:100%;overflow:hidden;white-space:nowrap;background-color:inherit}.breadcrumb-item:before{vertical-align:top}.log-visualizer-tooltip{max-width:300px}.ReactTable{display:table}